@@ -280,7 +280,7 @@ static int fio_netio_recv(struct thread_data *td, struct io_u *io_u)
 
 	do {
 		if (nd->net_protocol == IPPROTO_UDP) {
-			socklen_t len = sizeof(nd->addr);
+			fio_socklen_t len = sizeof(nd->addr);
 			struct sockaddr *from = (struct sockaddr *) &nd->addr;
 
 			ret = recvfrom(io_u->file->fd, io_u->xfer_buf,
@@ -377,7 +377,7 @@ static int fio_netio_connect(struct thread_data *td, struct fio_file *f)
 static int fio_netio_accept(struct thread_data *td, struct fio_file *f)
 {
 	struct netio_data *nd = td->io_ops->data;
-	socklen_t socklen = sizeof(nd->addr);
+	fio_socklen_t socklen = sizeof(nd->addr);
 
 	if (nd->net_protocol == IPPROTO_UDP) {
 		f->fd = nd->listenfd;
@@ -633,7 +633,7 @@ static struct ioengine_ops ioengine_splice = {
 	.open_file	= fio_netio_open_file,
 	.close_file	= generic_close_file,
 	.flags		= FIO_SYNCIO | FIO_DISKLESSIO | FIO_UNIDIR |
-			  FIO_SIGQUIT | FIO_PIPEIO,
+			  FIO_SIGTERM | FIO_PIPEIO,
 };
 #endif
 
@@ -648,7 +648,7 @@ static struct ioengine_ops ioengine_rw = {
 	.open_file	= fio_netio_open_file,
 	.close_file	= fio_netio_close_file,
 	.flags		= FIO_SYNCIO | FIO_DISKLESSIO | FIO_UNIDIR |
-			  FIO_SIGQUIT | FIO_PIPEIO,
+			  FIO_SIGTERM | FIO_PIPEIO,
 };
 
 static void fio_init fio_netio_register(void)
